# -*- coding: utf-8 -*-
import json
import datetime


class Alert(object):
    """
    Describe each alert raised by Chariot
    """
    def __init__(self, msg=None, severity=100):
        self.timestamp = datetime.datetime.now().isoformat()
        self.message = msg
        self.severity = severity

    def dict(self):
        return {
            'message': self.message,
            'severity': self.severity
        }

    def __str__(self):
        return self.__unicode__()

    def __unicode__(self):
        return json.dumps(self.dict())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import uuid
import gmqtt
from ..utilities import Tracer


class LocalConnector(object):
    def __init__(self):
        self.connack = None
        self.client = None
        self.tracer = None

        self.disconnected = False
        self.connected = False

    def clear(self):
        self.__init__()

    def subscribe(self, topic, qos=1):
        self.client.subscribe(topic, qos=qos)

    def publish(self, topic, msg, qos=1):
        self.client.publish(topic, msg, qos=qos)

    def on_message(self, client, topic, payload, qos, properties):
        pass

    def on_subscribe(self, client, mid, qos):
        pass

    def on_disconnect(self, client, packet):
        self.disconnected = True

    def on_connect(self, client, flags, rc, properties):
        self.connected = True
        self.connack = (flags, rc, properties)

    def register_for_client(self, client):
        client.on_disconnect = self.on_disconnect
        client.on_message = self.on_message
        client.on_connect = self.on_connect
        client.on_subscribe = self.on_subscribe

        self.client = client

    def inject_tracer(self, tracer):
        self.tracer = tracer

    def set_up_tracer(self, options):
        self.tracer = Tracer(options)
        self.tracer.init_tracer()

    def start_span(self, id, child_span=None):
        if self.tracer is None:
            return

        if child_span is None:
            return self.tracer.tracer.start_span(id)
        else:
            return self.tracer.tracer.start_span(id, child_of=child_span)

    def close_span(self, span):
        if self.tracer is None:
            return
        span.finish()


async def create_client(options, postfix='_client'):
    client_id = '%s%s' % (uuid.uuid4(), postfix)
  
    client = gmqtt.Client(client_id, clean_session=True)
    await client.connect(host=options['host'], port=options['port'], version=4)

    return client


# -*- coding: utf-8 -*-
import json
import datetime


class DataPointFactory(object):
    def __init__(self, db, table):
        self.db = db
        self.table = table

    def from_mqtt_message(self, message, attribute_name=None):
        msg = message.payload.decode('utf-8')
        message_parsed = self.from_json_string(msg, attribute_name)
        message_parsed.topic = message.topic
        return message_parsed

    def from_json_string(self, msg, attribute_name=None):
        decoded_msg = json.loads(msg)
        if attribute_name is None:
            message_payload = decoded_msg
        else:
            message_payload = decoded_msg[attribute_name]
        return DataPoint(self.db, self.table, message_payload)


class DataPoint(object):
    def __init__(self, db, table, message):
        self.db = db
        self.table = table
        self.message = message
        self.timestamp = datetime.datetime.now().isoformat()
        self.topic = None

    def _event_type(self):
        if self.topic is None:
            return ''
        return self.topic.replace('/', '.')
